@@ -5,43 +5,53 @@
  *
  * Clock
  */
+
+#define NEED_TIME_H
+
 #include "config.h"
 #include "ylib.h"
-#include "aclock.h"
+#include "sysdep.h"
 
-#include "yapp.h"
+#include "aclock.h"
+#include "wmtaskbar.h"
+#include "wpixmaps.h"
+#include "wmapp.h"
 #include "prefs.h"
-#include <string.h>
-#include <stdio.h>
 
 
 #ifdef CONFIG_APPLET_CLOCK
 
-YPixmap *PixNum[10] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
-YPixmap *PixSpace = 0;
-YPixmap *PixColon = 0;
-YPixmap *PixSlash = 0;
-YPixmap *PixA = 0;
-YPixmap *PixP = 0;
-YPixmap *PixM = 0;
-YPixmap *PixDot = 0;
-
 YColor *YClock::clockBg = 0;
 YColor *YClock::clockFg = 0;
-YFont *YClock::clockFont = 0;
+ref<YFont> YClock::clockFont;
 
-YClock::YClock(YWindow *aParent): YWindow(aParent) {
-    if (clockBg == 0)
+static YColor *taskBarBg = 0;
+static char const *AppletClockTimeFmt = "%T";
+
+inline char const * strTimeFmt(struct tm const & t) {
+    if ((ledPixColon == null) || (! prettyClock))
+        return (fmtTimeAlt && (t.tm_sec & 1) ? fmtTimeAlt : fmtTime);
+    return AppletClockTimeFmt;
+}
+
+YClock::YClock(YSMListener *smActionListener, YWindow *aParent): YWindow(aParent) {
+    this->smActionListener = smActionListener;
+    if (clockBg == 0 && *clrClock)
         clockBg = new YColor(clrClock);
     if (clockFg == 0)
         clockFg = new YColor(clrClockText);
-    if (clockFont == 0)
-        clockFont = YFont::getFont(clockFontName);
+    if (clockFont == null)
+        clockFont = YFont::getFont(XFA(clockFontName));
+    if (taskBarBg == 0) {
+        taskBarBg = new YColor(clrDefaultTaskBar);
+    }
 
     clockUTC = false;
     toolTipUTC = false;
+    transparent = -1;
 
     clockTimer = new YTimer(1000);
+    clockTimer->setFixed();
     clockTimer->setTimerListener(this);
     clockTimer->startTimer();
     autoSize();
@@ -57,7 +67,6 @@ void YClock::autoSize() {
     char s[64];
     time_t newTime = time(NULL);
     struct tm t = *localtime(&newTime);
-    int maxDay = -1;
     int maxMonth = -1;
     int maxWidth = -1;
 
@@ -67,7 +76,7 @@ void YClock::autoSize() {
 
         t.tm_mon = m;
 
-        len = strftime(s, sizeof(s), fmtTime, &t);
+        len = strftime(s, sizeof(s), strTimeFmt(t), &t);
         w = calcWidth(s, len);
         if (w > maxWidth) {
             maxMonth = m;
@@ -80,10 +89,9 @@ void YClock::autoSize() {
 
         t.tm_wday = dw;
 
-        len = strftime(s, sizeof(s), fmtTime, &t);
+        len = strftime(s, sizeof(s), strTimeFmt(t), &t);
         w = calcWidth(s, len);
         if (w > maxWidth) {
-            maxDay = dw;
             maxWidth = w;
         }
     }
@@ -111,14 +119,13 @@ void YClock::updateToolTip() {
     char s[128];
     time_t newTime = time(NULL);
     struct tm *t;
-    int len;
 
     if (toolTipUTC)
         t = gmtime(&newTime);
     else
         t = localtime(&newTime);
 
-    len = strftime(s, sizeof(s), fmtDate, t);
+    strftime(s, sizeof(s), fmtDate, t);
 
     setToolTip(s);
 }
@@ -135,22 +142,25 @@ void YClock::handleCrossing(const XCrossingEvent &crossing) {
     YWindow::handleCrossing(crossing);
 }
 
+#ifdef DEBUG
 static int countEvents = 0;
 extern int xeventcount;
+#endif
 
 void YClock::handleClick(const XButtonEvent &up, int count) {
     if (up.button == 1) {
-        if ((count % 2) == 0) {
-            if (clockCommand && clockCommand[0])
-                app->runCommand(clockCommand);
-        }
+        if (clockCommand && clockCommand[0] &&
+            (taskBarLaunchOnSingleClick ? count == 1 : !(count % 2)))
+            smActionListener->runCommandOnce(clockClassHint, clockCommand);
+#ifdef DEBUG
     } else if (up.button == 2) {
         if ((count % 2) == 0)
             countEvents = !countEvents;
+#endif
     }
 }
 
-void YClock::paint(Graphics &g, int /*x*/, int /*y*/, unsigned int /*width*/, unsigned int /*height*/) {
+void YClock::paint(Graphics &g, const YRect &/*r*/) {
     int x = width();
     char s[64];
     time_t newTime = time(NULL);
@@ -162,20 +172,44 @@ void YClock::paint(Graphics &g, int /*x*/, int /*y*/, unsigned int /*width*/, un
     else
         t = localtime(&newTime);
 
+#ifdef DEBUG
     if (countEvents)
         len = sprintf(s, "%d", xeventcount);
     else
-        len = strftime(s, sizeof(s), fmtTime, t);
+#endif
+        len = strftime(s, sizeof(s), strTimeFmt(*t), t);
+
+    
+    //clean backgroung first, so that it is possible
+    //to use transparent lcd pixmaps
+    if (hasTransparency()) {
+#ifdef CONFIG_GRADIENTS
+        ref<YImage> gradient(parent()->getGradient());
+    
+        if (gradient != null)
+            g.drawImage(gradient, this->x(), this->y(),
+                         width(), height(), 0, 0);
+        else 
+#endif
+        if (taskbackPixmap != null) {
+            g.fillPixmap(taskbackPixmap, 0, 0,
+                         width(), height(), this->x(), this->y());
+        }
+        else {
+            g.setColor(taskBarBg);
+            g.fillRect(0, 0, width(), height());
+        }
+    }
 
     if (prettyClock) {
         i = len - 1;
         for (i = len - 1; x >= 0; i--) {
-            YPixmap *p;
+            ref<YPixmap> p;
             if (i >= 0)
                 p = getPixmap(s[i]);
             else
-                p = PixSpace;
-            if (p) {
+                p = ledPixSpace;
+            if (p != null) {
                 x -= p->width();
                 g.drawPixmap(p, x, 0);
             } else if (i < 0) {
@@ -188,8 +222,11 @@ void YClock::paint(Graphics &g, int /*x*/, int /*y*/, unsigned int /*width*/, un
         int y =  (height() - 1 - clockFont->height()) / 2
             + clockFont->ascent();
 
-        g.setColor(clockBg);
-        g.fillRect(0, 0, width(), height());
+        if (clockBg) {
+            g.setColor(clockBg);
+            g.fillRect(0, 0, width(), height());
+        }
+
         g.setColor(clockFg);
         g.setFont(clockFont);
         g.drawChars(s, 0, len, 2, y);
@@ -206,8 +243,8 @@ bool YClock::handleTimer(YTimer *t) {
     return true;
 }
 
-YPixmap *YClock::getPixmap(char c) {
-    YPixmap *pix = 0;
+ref<YPixmap> YClock::getPixmap(char c) {
+    ref<YPixmap> pix;
     switch (c) {
     case '0':
     case '1':
@@ -219,31 +256,31 @@ YPixmap *YClock::getPixmap(char c) {
     case '7':
     case '8':
     case '9':
-        pix = PixNum[c - '0'];
+        pix = ledPixNum[c - '0'];
         break;
     case ' ':
-        pix = PixSpace;
+        pix = ledPixSpace;
         break;
     case ':':
-        pix = PixColon;
+        pix = ledPixColon;
         break;
     case '/':
-        pix = PixSlash;
+        pix = ledPixSlash;
         break;
     case '.':
-        pix = PixDot;
+        pix = ledPixDot;
         break;
     case 'p':
     case 'P':
-        pix = PixP;
+        pix = ledPixP;
         break;
     case 'a':
     case 'A':
-        pix = PixA;
+        pix = ledPixA;
         break;
     case 'm':
     case 'M':
-        pix = PixM;
+        pix = ledPixM;
         break;
     }
     return pix;
@@ -256,11 +293,29 @@ int YClock::calcWidth(const char *s, int count) {
         int len = 0;
 
         while (count--) {
-            YPixmap *p = getPixmap(*s++);
-            if (p)
+            ref<YPixmap> p = getPixmap(*s++);
+            if (p != null)
                 len += p->width();
         }
         return len;
     }
 }
+
+bool YClock::hasTransparency() {
+    if (transparent == 0)
+        return false;
+    else if (transparent == 1)
+        return true;
+    if (!prettyClock) {
+        transparent = 1;
+        return true;
+    }
+    ref<YPixmap> p = getPixmap('0');
+    if (p != null && p->mask()) {
+        transparent = 1;
+        return true;
+    }
+    transparent = 0;
+    return false;
+}
 #endif
