@@ -1,17 +1,27 @@
 /*
  * IceWM
  *
- * Copyright (C) 1997-2001 Marko Macek
+ * Copyright (C) 1997-2002 Marko Macek
  */
 #include "config.h"
+
 #include "yfull.h"
 #include "ykey.h"
 #include "wmframe.h"
 
 #include "wmclient.h"
 #include "wmstatus.h"
-#include "yapp.h"
+#include "yxapp.h"
 #include "wmapp.h"
+#include "yrect.h"
+#include "prefs.h"
+
+#include "intl.h"
+
+#include "wmtaskbar.h"
+#include "aworkspaces.h"
+
+#include <stdio.h>
 
 void YFrameWindow::snapTo(int &wx, int &wy,
                           int rx1, int ry1, int rx2, int ry2,
@@ -104,24 +114,24 @@ void YFrameWindow::snapTo(int &wx, int &wy) {
     int xp = wx, yp = wy;
     int rx1, ry1, rx2, ry2;
 
+    int mx, my, Mx, My;
+    manager->getWorkArea(this, &mx, &my, &Mx, &My, getScreen());
+
     /// !!! clean this up, it should snap to the closest thing it finds
-    
+
     // try snapping to the border first
     flags |= 4;
-    if (xp < manager->minX(getLayer()) || xp + int(width()) > manager->maxX(getLayer())) {
+    if (xp < mx || xp + int(width()) > Mx) {
         xp += borderX();
         flags |= 8;
     }
-    if (yp < manager->minY(getLayer()) || yp + int(height()) > manager->maxY(getLayer())) {
+    if (yp < my || yp + int(height()) > My) {
         yp += borderY();
         flags |= 16;
     }
-    snapTo(xp, yp,
-           manager->minX(getLayer()),
-           manager->minY(getLayer()),
-           manager->maxX(getLayer()),
-           manager->maxY(getLayer()),
-           flags);
+
+    snapTo(xp, yp, mx, my, Mx, My, flags);
+
     if (flags & 8) {
         xp -= borderX();
         flags &= ~8;
@@ -150,8 +160,10 @@ void YFrameWindow::snapTo(int &wx, int &wy) {
     flags &= ~4;
 
     if (flags & (1 | 2)) {
-        // we only snap to windows below, hope that's ok
-        while (f) {
+        for (; f; f = f->nextLayer()) {
+            if (affectsWorkArea() && f->inWorkArea())
+                continue;
+
             if (f != this && f->visible()) {
                 rx1 = f->x();
                 ry1 = f->y();
@@ -161,50 +173,50 @@ void YFrameWindow::snapTo(int &wx, int &wy) {
                 if (!(flags & (1 | 2)))
                     break;
             }
-            f = f->nextLayer();
         }
     }
     wx = xp;
     wy = yp;
 }
 
-void YFrameWindow::drawOutline(int x, int y, int w, int h) {
-    int bw = (wsBorderX + wsBorderY) / 2;
-    static GC outlineGC = None;
+void YFrameWindow::drawMoveSizeFX(int x, int y, int w, int h, bool) {
+    if ((movingWindow && opaqueMove) ||
+        (sizingWindow && opaqueResize))
+        return;
 
-    if (outlineGC == None) {
+    int const bw((wsBorderX + wsBorderY) / 2);
+    int const bo((wsBorderX + wsBorderY) / 4);
+    static Graphics * gc(NULL);
+
+    if (gc == NULL) {
         XGCValues gcv;
 
         gcv.foreground = YColor(clrActiveBorder).pixel();
         gcv.function = GXxor;
         gcv.graphics_exposures = False;
-        gcv.line_width = (wsBorderX + wsBorderY) / 2;
+        gcv.line_width = bw;
         gcv.subwindow_mode = IncludeInferiors;
 
-        outlineGC = XCreateGC(app->display(), desktop->handle(),
-                              GCForeground |
-                              GCFunction |
-                              GCGraphicsExposures |
-                              GCLineWidth |
-                              GCSubwindowMode,
-                              &gcv);
+        gc = new Graphics(*desktop, GCForeground | GCFunction |
+                                    GCGraphicsExposures | GCLineWidth |
+                                    GCSubwindowMode, &gcv);
     }
 
-    x += bw / 2;
-    y += bw / 2;
-    w -= bw;
-    h -= bw;
-    XDrawRectangle(app->display(), manager->handle(), outlineGC,
-                   x, y, w, h);
+    gc->drawRect(x + bo, y + bo, w - bw, h - bw);
 }
 
 int YFrameWindow::handleMoveKeys(const XKeyEvent &key, int &newX, int &newY) {
-    KeySym k = XKeycodeToKeysym(app->display(), key.keycode, 0);
+    KeySym k = keyCodeToKeySym(key.keycode);
     int m = KEY_MODMASK(key.state);
     int factor = 1;
 
-    if (m == ShiftMask)
+    int mx, my, Mx, My;
+    manager->getWorkArea(this, &mx, &my, &Mx, &My);
+
+    if (m & ShiftMask)
         factor = 4;
+    if (m & ControlMask)
+        factor<<= 4;
 
     if (k == XK_Left || k == XK_KP_Left)
         newX -= factor;
@@ -215,14 +227,17 @@ int YFrameWindow::handleMoveKeys(const XKeyEvent &key, int &newX, int &newY) {
     else if (k == XK_Down || k == XK_KP_Down)
         newY += factor;
     else if (k == XK_Home || k == XK_KP_Home)
-        newX = manager->minX(getLayer()) - borderX();
+        newX = mx - borderX();
     else if (k == XK_End || k == XK_KP_End)
-        newX = manager->maxX(getLayer()) - width() + borderX();
+        newX = Mx - width() + borderX();
     else if (k == XK_Prior || k == XK_KP_Prior)
-        newY = manager->minY(getLayer()) - borderY();
+        newY = my - borderY();
     else if (k == XK_Next || k == XK_KP_Next)
-        newY = manager->maxY(getLayer()) - height() + borderY();
-    else if (k == XK_Return || k == XK_KP_Enter)
+        newY = My - height() + borderY();
+    else if (k == XK_KP_Begin) {
+        newX = (mx + Mx - (int)width()) / 2;
+        newY = (my + My - (int)height()) / 2;
+    } else if (k == XK_Return || k == XK_KP_Enter)
         return -1;
     else if (k ==  XK_Escape) {
         newX = origX;
@@ -233,21 +248,26 @@ int YFrameWindow::handleMoveKeys(const XKeyEvent &key, int &newX, int &newY) {
     return 1;
 }
 
+/******************************************************************************/
+
 int YFrameWindow::handleResizeKeys(const XKeyEvent &key,
                                    int &newX, int &newY, int &newWidth, int &newHeight,
                                    int incX, int incY)
 {
-    KeySym k = XKeycodeToKeysym(app->display(), key.keycode, 0);
+    KeySym k = keyCodeToKeySym(key.keycode);
     int m = KEY_MODMASK(key.state);
     int factor = 1;
 
-    if (m == ShiftMask)
+    if (m & ShiftMask)
         factor = 4;
+    if (m & ControlMask)
+        factor<<= 4;
 
     if (k == XK_Left || k == XK_KP_Left) {
         if (grabX == 0) {
             grabX = -1;
-        } else if (grabX == 1) {
+        }
+        if (grabX == 1) {
             newWidth -= incX * factor;
         } else if (grabX == -1) {
             newWidth += incX * factor;
@@ -256,7 +276,8 @@ int YFrameWindow::handleResizeKeys(const XKeyEvent &key,
     } else if (k == XK_Right || k == XK_KP_Right) {
         if (grabX == 0) {
             grabX = 1;
-        } else if (grabX == 1) {
+        }
+        if (grabX == 1) {
             newWidth += incX * factor;
         } else if (grabX == -1) {
             newWidth -= incX * factor;
@@ -265,7 +286,8 @@ int YFrameWindow::handleResizeKeys(const XKeyEvent &key,
     } else if (k == XK_Up || k == XK_KP_Up) {
         if (grabY == 0) {
             grabY = -1;
-        } else if (grabY == 1) {
+        }
+        if (grabY == 1) {
             newHeight -= incY * factor;
         } else if (grabY == -1) {
             newHeight += incY * factor;
@@ -274,7 +296,8 @@ int YFrameWindow::handleResizeKeys(const XKeyEvent &key,
     } else if (k == XK_Down || k == XK_KP_Down) {
         if (grabY == 0) {
             grabY = 1;
-        } else if (grabY == 1) {
+        }
+        if (grabY == 1) {
             newHeight += incY * factor;
         } else if (grabY == -1) {
             newHeight -= incY * factor;
@@ -297,7 +320,7 @@ void YFrameWindow::handleMoveMouse(const XMotionEvent &motion, int &newX, int &n
     int mouseX = motion.x_root;
     int mouseY = motion.y_root;
 
-    constrainMouseToWorkspace(mouseX, mouseY);
+    //constrainMouseToWorkspace(mouseX, mouseY);
 
     newX = mouseX - buttonDownX;
     newY = mouseY - buttonDownY;
@@ -308,37 +331,48 @@ void YFrameWindow::handleMoveMouse(const XMotionEvent &motion, int &newX, int &n
     newY += borderY();
     int n = -2;
 
-    if (!(motion.state & app->AltMask)) {
-        if (EdgeResistance == 10000) {
-            if (newX + int(width() + n * borderX()) > manager->maxX(getLayer()))
-                newX = manager->maxX(getLayer()) - width() - n * borderX();
-            if (newY + int(height() + n * borderY()) > manager->maxY(getLayer()))
-                newY = manager->maxY(getLayer()) - height() - n * borderY();
-            if (newX < manager->minX(getLayer()))
-                newX = manager->minX(getLayer());
-            if (newY < manager->minY(getLayer()))
-                newY = manager->minY(getLayer());
-        } else if (/*EdgeResistance >= 0 && %%% */ EdgeResistance < 10000) {
-            if (newX + int(width() + n * borderX()) > manager->maxX(getLayer()))
-                if (newX + int(width() + n * borderX()) < int(manager->maxX(getLayer()) + EdgeResistance))
-                    newX = manager->maxX(getLayer()) - width() - n * borderX();
+    int mx, my, Mx, My;
+    manager->getWorkArea(this, &mx, &my, &Mx, &My);
+
+
+    if (!(motion.state & ShiftMask)) {
+        if (/*EdgeResistance >= 0 && %%% */ EdgeResistance < 10000) {
+            if (newX + int(width() + n * borderX()) > Mx) {
+                if (newX + int(width() + n * borderX()) < int(Mx + EdgeResistance))
+                    newX = Mx - width() - n * borderX();
                 else if (motion.state & ShiftMask)
                     newX -= EdgeResistance;
-            if (newY + int(height() + n * borderY()) > manager->maxY(getLayer()))
-                if (newY + int(height() + n * borderY()) < int(manager->maxY(getLayer()) + EdgeResistance))
-                    newY = manager->maxY(getLayer()) - height() - n * borderY();
+            }
+            if (newY + int(height() + n * borderY()) > My) {
+                if (newY + int(height() + n * borderY()) < int(My + EdgeResistance))
+                    newY = My - height() - n * borderY();
                 else if (motion.state & ShiftMask)
                     newY -= EdgeResistance;
-            if (newX < manager->minX(getLayer()))
-                if (newX > int(- EdgeResistance + manager->minX(getLayer())))
-                    newX = manager->minX(getLayer());
+            }
+            if (newX < mx) {
+                if (newX > int(- EdgeResistance + mx))
+                    newX = mx;
                 else if (motion.state & ShiftMask)
                     newX += EdgeResistance;
-            if (newY < manager->minY(getLayer()))
-                if (newY > int(- EdgeResistance + manager->minY(getLayer())))
-                    newY = manager->minY(getLayer());
+            }
+            if (newY < my) {
+                if (newY > int(- EdgeResistance + my))
+                    newY = my;
                 else if (motion.state & ShiftMask)
                     newY += EdgeResistance;
+            }
+        }
+        if (EdgeResistance == 10000 || isMaximizedHoriz()) {
+            if (newX + int(width() + n * borderX()) > Mx)
+                newX = Mx - width() - n * borderX();
+            if (newX < mx)
+                newX = mx;
+        }
+        if (EdgeResistance == 10000 || isMaximizedVert()) {
+            if (newY + int(height() + n * borderY()) > My)
+                newY = My - height() - n * borderY();
+            if (newY < my)
+                newY = my;
         }
     }
     newX -= borderX();
@@ -351,7 +385,7 @@ void YFrameWindow::handleResizeMouse(const XMotionEvent &motion,
     int mouseX = motion.x_root;
     int mouseY = motion.y_root;
 
-    constrainMouseToWorkspace(mouseX, mouseY);
+//    constrainMouseToWorkspace(mouseX, mouseY);
 
     if (grabX == -1) {
         newX = mouseX - buttonDownX;
@@ -370,9 +404,10 @@ void YFrameWindow::handleResizeMouse(const XMotionEvent &motion,
     newWidth -= 2 * borderX();
     newHeight -= 2 * borderY() + titleY();
     client()->constrainSize(newWidth, newHeight,
+                            ///getLayer(),
                             YFrameClient::csRound |
-                            (grabX ? YFrameClient::csKeepX : 0) |
-                            (grabY ? YFrameClient::csKeepY : 0));
+                            ((grabX != 0) ? YFrameClient::csKeepX : 0) |
+                            ((grabY != 0) ? YFrameClient::csKeepY : 0));
     newWidth += 2 * borderX();
     newHeight += 2 * borderY() + titleY();
 
@@ -384,159 +419,169 @@ void YFrameWindow::handleResizeMouse(const XMotionEvent &motion,
 }
 
 void YFrameWindow::outlineMove() {
-    int xx = x(), yy = y();
-    unsigned int modifiers = 0;
+    int xx(x()), yy(y());
 
-    XGrabServer(app->display());
-    XSync(app->display(), False);
-    drawOutline(xx, yy, width(), height());
-    while (1) {
+    XGrabServer(xapp->display());
+    XSync(xapp->display(), False);
+
+    for(;;) {
         XEvent xev;
 
-        XWindowEvent(app->display(),
-                     handle(),
-                     KeyPressMask | ExposureMask |
-                     ButtonPressMask | ButtonReleaseMask | PointerMotionMask, &xev);
+        XWindowEvent(xapp->display(), handle(),
+                     KeyPressMask | ExposureMask | 
+                     ButtonPressMask | ButtonReleaseMask | 
+                     PointerMotionMask, &xev);
+
         switch (xev.type) {
-        case KeyPress:
-            modifiers = xev.xkey.state;
-            {
-                int ox = xx;
-                int oy = yy;
+            case KeyPress: {
+
+                int const ox(xx), oy(yy);
                 int r;
 
                 switch (r = handleMoveKeys(xev.xkey, xx, yy)) {
-                case 0:
-                    break;
-                case 1:
-                case -2:
-                    if (xx != ox || yy != oy) {
-                        drawOutline(ox, oy, width(), height());
+                    case 1:
+                    case -2:
+                        if (xx != ox || yy != oy) {
+                            drawMoveSizeFX(ox, oy, width(), height());
 #ifndef LITE
-                        statusMoveSize->setStatus(this, xx, yy, width(), height());
+                            statusMoveSize->setStatus(this, YRect(xx, yy, width(), height()));
 #endif
-                        drawOutline(xx, yy, width(), height());
-                    }
-                    if (r == -2)
+                            drawMoveSizeFX(xx, yy, width(), height());
+                        }
+
+                        if (r == -2)
+                            goto end;
+
+                        break;
+
+                    case 0:
+                        break;
+
+                    case -1:
                         goto end;
-                    break;
-                case -1:
-                    goto end;
                 }
+
+                break;
             }
-            break;
-        case ButtonPress:
-        case ButtonRelease:
-            modifiers = xev.xbutton.state;
-            goto end;
-        case MotionNotify:
-            {
-                int ox = xx;
-                int oy = yy;
+
+            case ButtonPress:
+            case ButtonRelease:
+                goto end;
+
+            case MotionNotify: {
+                int const ox(xx), oy(yy);
 
                 handleMoveMouse(xev.xmotion, xx, yy);
+
                 if (xx != ox || yy != oy) {
+                    drawMoveSizeFX(ox, oy, width(), height());
 #ifndef LITE
-                    statusMoveSize->setStatus(this, xx, yy, width(), height());
+                    statusMoveSize->setStatus(this, YRect(xx, yy, width(), height()));
 #endif
-                    drawOutline(ox, oy, width(), height());
-                    drawOutline(xx, yy, width(), height());
+                    drawMoveSizeFX(xx, yy, width(), height());
                 }
+
+                break;
             }
-            break;
         }
     }
+
 end:
-    drawOutline(xx, yy, width(), height());
-    XSync(app->display(), False);
+    drawMoveSizeFX(xx, yy, width(), height());
+
+    XSync(xapp->display(), False);
     moveWindow(xx, yy);
-    XUngrabServer(app->display());
+    XUngrabServer(xapp->display());
 }
 
 void YFrameWindow::outlineResize() {
-    int xx = x(), yy = y(), ww = width(), hh = height();
-    int incX = 1;
-    int incY = 1;
+    int xx(x()), yy(y()), ww(width()), hh(height());
+    int incX(1), incY(1);
 
     if (client()->sizeHints()) {
         incX = client()->sizeHints()->width_inc;
         incY = client()->sizeHints()->height_inc;
     }
 
-    XGrabServer(app->display());
-    XSync(app->display(), False);
-    drawOutline(xx, yy, ww, hh);
-    while (1) {
+    XGrabServer(xapp->display());
+    XSync(xapp->display(), False);
+
+    for(;;) {
         XEvent xev;
 
-        XWindowEvent(app->display(),
-                     handle(),
+        XWindowEvent(xapp->display(), handle(),
                      KeyPressMask | ExposureMask |
-                     ButtonPressMask | ButtonReleaseMask | PointerMotionMask, &xev);
+                     ButtonPressMask | ButtonReleaseMask | 
+                     PointerMotionMask, &xev);
+
         switch (xev.type) {
-        case KeyPress:
-            {
-                int ox = xx;
-                int oy = yy;
-                int ow = ww;
-                int oh = hh;
+            case KeyPress: {
+                int const ox(xx), oy(yy), ow(ww), oh(hh);
                 int r;
 
-                switch (r = handleResizeKeys(xev.xkey, xx, yy, ww, hh, incX, incY)) {
-                case 0:
-                    break;
-                case -2:
-                case 1:
-                    if (ox != xx || oy != yy || ow != ww || oh != hh) {
-                        drawOutline(ox, oy, ow, oh);
+                switch (r = handleResizeKeys(xev.xkey, xx, yy, ww, hh,
+                                             incX, incY)) {
+                    case -2:
+                    case 1:
+                        if (ox != xx || oy != yy || ow != ww || oh != hh) {
+                            drawMoveSizeFX(ox, oy, ow, oh);
 #ifndef LITE
-                        statusMoveSize->setStatus(this, xx, yy, ww, hh);
+                            statusMoveSize->setStatus(this, YRect(xx, yy, ww, hh));
 #endif
-                        drawOutline(xx, yy, ww, hh);
-                    }
-                    if (r == -2)
+                            drawMoveSizeFX(xx, yy, ww, hh);
+                        }
+
+                        if (r == -2)
+                            goto end;
+
+                        break;
+
+                    case 0:
+                        break;
+
+                    case -1:
                         goto end;
-                    break;
-                case -1:
-                    goto end;
                 }
+                
+                break;
             }
-            break;
-        case ButtonPress:
-        case ButtonRelease:
-            goto end;
-        case MotionNotify:
-            {
-                int ox = xx;
-                int oy = yy;
-                int ow = ww;
-                int oh = hh;
+
+            case ButtonPress:
+            case ButtonRelease:
+                goto end;
+
+            case MotionNotify: {
+                int const ox(xx), oy(yy), ow(ww), oh(hh);
 
                 handleResizeMouse(xev.xmotion, xx, yy, ww,hh);
+
                 if (ox != xx || oy != yy || ow != ww || oh != hh) {
-                    drawOutline(ox, oy, ow, oh);
+                    drawMoveSizeFX(ox, oy, ow, oh);
 #ifndef LITE
-                    statusMoveSize->setStatus(this, xx, yy, ww, hh);
+                    statusMoveSize->setStatus(this, YRect(xx, yy, ww, hh));
 #endif
-                    drawOutline(xx, yy, ww, hh);
+                    drawMoveSizeFX(xx, yy, ww, hh);
                 }
+
+                break;
             }
-            break;
         }
     }
+
 end:
-    drawOutline(xx, yy, ww, hh);
-    XSync(app->display(), False);
-    setGeometry(xx, yy, ww, hh);
-    XUngrabServer(app->display());
+    drawMoveSizeFX(xx, yy, ww, hh);
+
+    XSync(xapp->display(), False);
+    setCurrentGeometryOuter(YRect(xx, yy, ww, hh));
+    XUngrabServer(xapp->display());
 }
 
 void YFrameWindow::manualPlace() {
-    int xx = x(), yy = y();
-    Cursor grabPointer = movePointer;
+    int xx(x()), yy(y());
+
+    grabX = 1;
+    grabY = 1;
 
-    grabX = borderX();
-    grabY = borderY();
     origX = x();
     origY = y();
     origW = width();
@@ -544,78 +589,91 @@ void YFrameWindow::manualPlace() {
     buttonDownX = 0;
     buttonDownY = 0;
 
-    if (!app->grabEvents(desktop, grabPointer,
-                         ButtonPressMask |
-                         ButtonReleaseMask |
-                         PointerMotionMask))
-        return ;
-    XGrabServer(app->display());
+    if (!xapp->grabEvents(desktop,
+                          YXApplication::movePointer.handle(),
+                          ButtonPressMask |
+                          ButtonReleaseMask |
+                          PointerMotionMask))
+        return;
+
+    XGrabServer(xapp->display());
 #ifndef LITE
     statusMoveSize->begin(this);
 #endif
-    drawOutline(xx, yy, width(), height());
-    while (1) {
+
+    drawMoveSizeFX(xx, yy, width(), height());
+
+    for(;;) {
         XEvent xev;
 
-        XMaskEvent(app->display(),
+        XMaskEvent(xapp->display(),
                    KeyPressMask |
                    ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
                    &xev);
+
         switch (xev.type) {
-        case KeyPress:
-            {
-                int ox = xx;
-                int oy = yy;
+            case KeyPress: {
+                int const ox(xx), oy(yy);
+
                 int r;
 
                 switch (r = handleMoveKeys(xev.xkey, xx, yy)) {
-                case 0:
-                    break;
-                case 1:
-                case -2:
-                    if (xx != ox || yy != oy) {
-                        drawOutline(ox, oy, width(), height());
+                    case 1:
+                    case -2:
+                        if (xx != ox || yy != oy) {
+                            drawMoveSizeFX(ox, oy, width(), height());
 #ifndef LITE
-                        statusMoveSize->setStatus(this, xx, yy, width(), height());
+                            statusMoveSize->setStatus(this, YRect(xx, yy, width(), height()));
 #endif
-                        drawOutline(xx, yy, width(), height());
-                    }
-                    if (r == -2)
+                            drawMoveSizeFX(xx, yy, width(), height());
+                        }
+
+                        if (r == -2)
+                            goto end;
+
+                        break;
+
+                    case 0:
+                        break;
+
+                    case -1:
                         goto end;
-                    break;
-                case -1:
-                    goto end;
                 }
+
+                break;
             }
-            break;
-        case ButtonPress:
-        case ButtonRelease:
-            goto end;
-        case MotionNotify:
-            {
-                int ox = xx;
-                int oy = yy;
+            
+
+            case ButtonPress:
+            case ButtonRelease:
+                goto end;
+
+            case MotionNotify: {
+                int const ox(xx), oy(yy);
 
                 handleMoveMouse(xev.xmotion, xx, yy);
                 if (xx != ox || yy != oy) {
+                    drawMoveSizeFX(ox, oy, width(), height());
 #ifndef LITE
-                    statusMoveSize->setStatus(this, xx, yy, width(), height());
+                    statusMoveSize->setStatus(this, YRect(xx, yy, width(), height()));
 #endif
-                    drawOutline(ox, oy, width(), height());
-                    drawOutline(xx, yy, width(), height());
+                    drawMoveSizeFX(xx, yy, width(), height());
                 }
+
+                break;
             }
-            break;
         }
     }
+
 end:
-    drawOutline(xx, yy, width(), height());
+    drawMoveSizeFX(xx, yy, width(), height());
+
 #ifndef LITE
     statusMoveSize->end();
 #endif
     moveWindow(xx, yy);
-    app->releaseEvents();
-    XUngrabServer(app->display());
+    xapp->releaseEvents();
+    XUngrabServer(xapp->display());
 }
 
 bool YFrameWindow::handleKey(const XKeyEvent &key) {
@@ -627,7 +685,7 @@ bool YFrameWindow::handleKey(const XKeyEvent &key) {
             switch (handleMoveKeys(key, newX, newY)) {
             case -2:
                 moveWindow(newX, newY);
-                /* nobreak */
+                /* fall-through */
             case -1:
                 endMoveSize();
                 break;
@@ -654,38 +712,41 @@ bool YFrameWindow::handleKey(const XKeyEvent &key) {
             case 0:
                 break;
             case 1:
-                newWidth -= 2 * borderX();
-                newHeight -= 2 * borderY() + titleY();
+                newWidth -= 2 * borderXN();
+                newHeight -= 2 * borderYN() + titleYN();
                 client()->constrainSize(newWidth, newHeight,
+                                        ///getLayer(),
                                         YFrameClient::csRound |
                                         (grabX ? YFrameClient::csKeepX : 0) |
                                         (grabY ? YFrameClient::csKeepY : 0));
-                newWidth += 2 * borderX();
-                newHeight += 2 * borderY() + titleY();
+                newWidth += 2 * borderXN();
+                newHeight += 2 * borderYN() + titleYN();
 
                 if (grabX == -1)
                     newX = x() + width() - newWidth;
                 if (grabY == -1)
                     newY = y() + height() - newHeight;
 
-                setGeometry(newX,
-                            newY,
-                            newWidth,
-                            newHeight);
+                drawMoveSizeFX(x(), y(), width(), height());
+                setCurrentGeometryOuter(YRect(newX, newY, newWidth, newHeight));
+                drawMoveSizeFX(x(), y(), width(), height());
 
 #ifndef LITE
                 statusMoveSize->setStatus(this);
 #endif
                 break;
             case -2:
-                setGeometry(newX, newY, newWidth, newHeight);
-                /* nobreak */
+                drawMoveSizeFX(x(), y(), width(), height());
+                setCurrentGeometryOuter(YRect(newX, newY, newWidth, newHeight));
+                drawMoveSizeFX(x(), y(), width(), height());
+                /* falls-through */
+
             case -1:
                 endMoveSize();
                 break;
             }
-        } else if (app->AltMask != 0) {
-            KeySym k = XKeycodeToKeysym(app->display(), key.keycode, 0);
+        } else if (xapp->AltMask != 0) {
+            KeySym k = keyCodeToKeySym(key.keycode);
             unsigned int m = KEY_MODMASK(key.state);
             unsigned int vm = VMod(m);
 
@@ -699,6 +760,8 @@ bool YFrameWindow::handleKey(const XKeyEvent &key) {
                 wmPrevWindow();
             } else if (IS_WMKEY(k, vm, gKeyWinMaximizeVert)) {
                 wmMaximizeVert();
+            } else if (IS_WMKEY(k, vm, gKeyWinMaximizeHoriz)) {
+                wmMaximizeHorz();
             } else if (IS_WMKEY(k, vm, gKeyWinRaise)) {
                 if (canRaise()) wmRaise();
             } else if (IS_WMKEY(k, vm, gKeyWinOccupyAll)) {
@@ -721,14 +784,57 @@ bool YFrameWindow::handleKey(const XKeyEvent &key) {
                 if (canHide()) wmHide();
             } else if (IS_WMKEY(k, vm, gKeyWinRollup)) {
                 if (canRollup()) wmRollup();
+            } else if (IS_WMKEY(k, vm, gKeyWinFullscreen)) {
+                if (canFullscreen()) wmToggleFullscreen();
             } else if (IS_WMKEY(k, vm, gKeyWinMenu)) {
-                popupSystemMenu();
-            }
-            if (isIconic() || isRollup()) {
+                popupSystemMenu(this);
+            } else if (IS_WMKEY(k, vm, gKeyWinArrangeN)) {
+                if (canMove()) wmArrange(waTop, waCenter);
+            } else if (IS_WMKEY(k, vm, gKeyWinArrangeNE)) {
+                if (canMove()) wmArrange(waTop, waRight);
+            } else if (IS_WMKEY(k, vm, gKeyWinArrangeE)) {
+                if (canMove()) wmArrange(waCenter, waRight);
+            } else if (IS_WMKEY(k, vm, gKeyWinArrangeSE)) {
+                if (canMove()) wmArrange(waBottom, waRight);
+            } else if (IS_WMKEY(k, vm, gKeyWinArrangeS)) {
+                if (canMove()) wmArrange(waBottom, waCenter);
+            } else if (IS_WMKEY(k, vm, gKeyWinArrangeSW)) {
+                if (canMove()) wmArrange(waBottom, waLeft);
+            } else if (IS_WMKEY(k, vm, gKeyWinArrangeW)) {
+                if (canMove()) wmArrange(waCenter, waLeft);
+            } else if (IS_WMKEY(k, vm, gKeyWinArrangeNW)) {
+                if (canMove()) wmArrange(waTop, waLeft);
+            } else if (IS_WMKEY(k, vm, gKeyWinArrangeC)) {
+                if (canMove()) wmArrange(waCenter, waCenter);
+            } else if (IS_WMKEY(k, vm, gKeyWinSnapMoveN)) {
+                if (canMove()) wmSnapMove(waTop, waCenter);
+            } else if (IS_WMKEY(k, vm, gKeyWinSnapMoveNE)) {
+                if (canMove()) wmSnapMove(waTop, waRight);
+            } else if (IS_WMKEY(k, vm, gKeyWinSnapMoveE)) {
+                if (canMove()) wmSnapMove(waCenter, waRight);
+            } else if (IS_WMKEY(k, vm, gKeyWinSnapMoveSE)) {
+                if (canMove()) wmSnapMove(waBottom, waRight);
+            } else if (IS_WMKEY(k, vm, gKeyWinSnapMoveS)) {
+                if (canMove()) wmSnapMove(waBottom, waCenter);
+            } else if (IS_WMKEY(k, vm, gKeyWinSnapMoveSW)) {
+                if (canMove()) wmSnapMove(waBottom, waLeft);
+            } else if (IS_WMKEY(k, vm, gKeyWinSnapMoveW)) {
+                if (canMove()) wmSnapMove(waCenter, waLeft);
+            } else if (IS_WMKEY(k, vm, gKeyWinSnapMoveNW)) {
+                if (canMove()) wmSnapMove(waTop, waLeft);
+            } else if (IS_WMKEY(k, vm, gKeyWinSmartPlace)) {
+                if (canMove()) {
+                    int newX = x();
+                    int newY = y();
+                    if (manager->getSmartPlace(true, this, newX, newY, width(), height(), getScreen())) {
+                        setCurrentPositionOuter(newX, newY);
+                    }
+                }
+            } else if (isIconic() || isRollup()) {
                 if (k == XK_Return || k == XK_KP_Enter) {
                     wmRestore();
                 } else if ((k == XK_Menu) || (k == XK_F10 && m == ShiftMask)) {
-                    popupSystemMenu();
+                    popupSystemMenu(this);
                 }
             }
         }
@@ -741,10 +847,6 @@ void YFrameWindow::constrainPositionByModifier(int &x, int &y, const XMotionEven
 
     x += borderX();
     y += borderY();
-    if (mask == ShiftMask) {
-        x = x / 4 * 4;
-        y = y / 4 * 4;
-    }
     x -= borderX();
     y -= borderY();
 
@@ -754,23 +856,20 @@ void YFrameWindow::constrainPositionByModifier(int &x, int &y, const XMotionEven
 }
 
 void YFrameWindow::constrainMouseToWorkspace(int &x, int &y) {
-    if (x < manager->minX(getLayer()))
-        x = manager->minX(getLayer());
-    else if (x >= manager->maxX(getLayer()))
-        x = manager->maxX(getLayer()) - 1;
-    if (y < manager->minY(getLayer()))
-        y = manager->minY(getLayer());
-    else if (y >= manager->maxY(getLayer()))
-        y = manager->maxY(getLayer()) - 1;
+    int mx, my, Mx, My;
+    manager->getWorkArea(this, &mx, &my, &Mx, &My);
+
+    x = clamp(x, mx, Mx - 1);
+    y = clamp(y, my, My - 1);
 }
 
 bool YFrameWindow::canSize(bool horiz, bool vert) {
     if (isRollup())
         return false;
-#ifndef NO_MWM_HINTS
-    if (!(client()->mwmFunctions() & MWM_FUNC_RESIZE))
+    if (isFullscreen())
+        return false;
+    if (!(frameFunctions() & ffResize))
         return false;
-#endif
     if (!sizeMaximized) {
         if ((!vert || isMaximizedVert()) &&
             (!horiz || isMaximizedHoriz()))
@@ -780,17 +879,34 @@ bool YFrameWindow::canSize(bool horiz, bool vert) {
 }
 
 bool YFrameWindow::canMove() {
-#ifndef NO_MWM_HINTS
-    if (!(client()->mwmFunctions() & MWM_FUNC_MOVE))
+    if (!(frameFunctions() & ffMove))
         return false;
-#endif
     return true;
 }
 
+#ifdef WMSPEC_HINTS
+void YFrameWindow::startMoveSize(int x, int y,
+                                 int direction)
+{
+    int sx[] = { -1, 0, 1, 1, 1, 0, -1, -1, 0 };
+    int sy[] = { -1, -1, -1, 0, 1, 1, 1, 0, 0 };
+
+    if (direction >= 0 && direction < (int) ACOUNT(sx)) {
+        MSG(("move size %d %d %d", x, y, direction));
+        if (direction == _NET_WM_MOVERESIZE_MOVE) {
+            x -= this->x();
+            y -= this->y();
+        }
+        startMoveSize((direction == _NET_WM_MOVERESIZE_MOVE) ? 1 : 0,
+                      true, sx[direction], sy[direction], x, y);
+    } else
+        warn(_("Unknown direction in move/resize request: %d"), direction);
+}
+#endif
+
 void YFrameWindow::startMoveSize(int doMove, int byMouse,
                                  int sideX, int sideY,
-                                 int mouseXroot, int mouseYroot)
-{
+                                 int mouseXroot, int mouseYroot) {
     Cursor grabPointer = None;
 
     sizeByMouse = byMouse;
@@ -801,39 +917,42 @@ void YFrameWindow::startMoveSize(int doMove, int byMouse,
     origW = width();
     origH = height();
 
+    manager->setWorkAreaMoveWindows(true);
     if (doMove && grabX == 0 && grabY == 0) {
         buttonDownX = mouseXroot;
         buttonDownY = mouseYroot;
 
 #ifdef CONFIG_GUIEVENTS
-	wmapp->signalGuiEvent(geWindowMoved);
+        wmapp->signalGuiEvent(geWindowMoved);
 #endif
-        grabPointer = movePointer;
+        grabPointer = YXApplication::movePointer.handle();
     } else if (!doMove) {
-        Cursor ptr = leftPointer;
 #ifdef CONFIG_GUIEVENTS
-	wmapp->signalGuiEvent(geWindowSized);
+        wmapp->signalGuiEvent(geWindowSized);
 #endif
 
         if (grabY == -1) {
             if (grabX == -1)
-                ptr = sizeTopLeftPointer;
+                grabPointer = YWMApp::sizeTopLeftPointer.handle();
             else if (grabX == 1)
-                ptr = sizeTopRightPointer;
+                grabPointer = YWMApp::sizeTopRightPointer.handle();
             else
-                ptr = sizeTopPointer;
+                grabPointer = YWMApp::sizeTopPointer.handle();
         } else if (grabY == 1) {
             if (grabX == -1)
-                ptr = sizeBottomLeftPointer;
+                grabPointer = YWMApp::sizeBottomLeftPointer.handle();
             else if (grabX == 1)
-                ptr = sizeBottomRightPointer;
+                grabPointer = YWMApp::sizeBottomRightPointer.handle();
             else
-                ptr = sizeBottomPointer;
+                grabPointer = YWMApp::sizeBottomPointer.handle();
         } else {
             if (grabX == -1)
-                ptr = sizeLeftPointer;
+                grabPointer = YWMApp::sizeLeftPointer.handle();
             else if (grabX == 1)
-                ptr = sizeRightPointer;
+                grabPointer = YWMApp::sizeRightPointer.handle();
+            else
+                grabPointer = YXApplication::leftPointer.handle();
+            
         }
 
         if (grabX == 1)
@@ -845,15 +964,14 @@ void YFrameWindow::startMoveSize(int doMove, int byMouse,
             buttonDownY = y() + height() - mouseYroot;
         else if (grabY == -1)
             buttonDownY = mouseYroot - y();
-
-        grabPointer = ptr;
     }
 
-    XSync(app->display(), False);
-    if (!app->grabEvents(this, grabPointer,
-                         ButtonPressMask |
-                         ButtonReleaseMask |
-                         PointerMotionMask))
+    XSync(xapp->display(), False);
+    if (!xapp->grabEvents(this,
+                          grabPointer,
+                          ButtonPressMask |
+                          ButtonReleaseMask |
+                          PointerMotionMask))
     {
         return ;
     }
@@ -866,6 +984,9 @@ void YFrameWindow::startMoveSize(int doMove, int byMouse,
 #ifndef LITE
     statusMoveSize->begin(this);
 #endif
+
+    drawMoveSizeFX(x(), y(), width(), height());
+
     if (doMove && !opaqueMove) {
         outlineMove();
         endMoveSize();
@@ -876,12 +997,25 @@ void YFrameWindow::startMoveSize(int doMove, int byMouse,
 }
 
 void YFrameWindow::endMoveSize() {
-    app->releaseEvents();
+    xapp->releaseEvents();
 #ifndef LITE
     statusMoveSize->end();
 #endif
+
+    if ((movingWindow && opaqueMove) ||
+        (sizingWindow && opaqueResize))
+        drawMoveSizeFX(x(), y(), width(), height());
+
     movingWindow = 0;
     sizingWindow = 0;
+
+    manager->setWorkAreaMoveWindows(false);
+
+#ifdef CONFIG_TASKBAR
+    if (taskBar && taskBar->workspacesPane()) {
+        taskBar->workspacesPane()->repaint();
+    }
+#endif
 }
 
 void YFrameWindow::handleBeginDrag(const XButtonEvent &down, const XMotionEvent &motion) {
@@ -902,12 +1036,12 @@ void YFrameWindow::handleBeginDrag(const XButtonEvent &down, const XMotionEvent
 
         if (grabY != 0 && grabX == 0) {
             if (down.x < int(wsCornerX)) grabX = -1;
-            else if (width() - down.x <= wsCornerX) grabX = 1;
+            else if ((int)width() - down.x <= wsCornerX) grabX = 1;
         }
 
         if (grabX != 0 && grabY == 0) {
             if (down.y < int(wsCornerY)) grabY = -1;
-            else if (height() - down.y <= wsCornerY) grabY = 1;
+            else if ((int)height() - down.y <= wsCornerY) grabY = 1;
         }
 
         if (grabX != 0 || grabY != 0) {
@@ -920,19 +1054,26 @@ void YFrameWindow::handleBeginDrag(const XButtonEvent &down, const XMotionEvent
 }
 
 void YFrameWindow::moveWindow(int newX, int newY) {
-    if (newX >= int(manager->maxX(getLayer()) - borderX()))
-        newX = manager->maxX(getLayer()) - borderX();
+/// TODO #warning "reevaluate if this is legacy"
+#if 0
+    if (!doNotCover()) {
+        int mx, my, Mx, My;
+        manager->getWorkArea(this, &mx, &my, &Mx, &My);
 
-    if (newY >= int(manager->maxY(getLayer()) - borderY() - titleY()))
-        newY = manager->maxY(getLayer()) - borderY() - titleY();
+        newX = clamp(newX, (int)(mx + borderX() - width()),
+                           (int)(Mx - borderX()));
+        newY = clamp(newY, (int)(my + borderY() - height()),
+                           (int)(My - borderY()));
+    }
+#endif
 
-    if (newX < int(manager->minX(getLayer()) - (width() - borderX())))
-        newX = manager->minX(getLayer()) - (width() - borderX());
+    if (opaqueMove)
+        drawMoveSizeFX(x(), y(), width(), height());
 
-    if (newY < int(manager->minY(getLayer()) - (height() - borderY())))
-        newY = manager->minY(getLayer()) - (height() - borderY());
+    setCurrentPositionOuter(newX, newY);
 
-    setPosition(newX, newY);
+    if (opaqueMove)
+        drawMoveSizeFX(x(), y(), width(), height());
 
 #ifndef LITE
     statusMoveSize->setStatus(this);
@@ -968,10 +1109,10 @@ void YFrameWindow::handleMotion(const XMotionEvent &motion) {
 
         handleResizeMouse(motion, newX, newY, newWidth, newHeight);
 
-        setGeometry(newX,
-                    newY,
-                    newWidth,
-                    newHeight);
+        drawMoveSizeFX(x(), y(), width(), height());
+        setCurrentGeometryOuter(YRect(newX, newY, newWidth, newHeight));
+        drawMoveSizeFX(x(), y(), width(), height());
+
 #ifndef LITE
         statusMoveSize->setStatus(this);
 #endif
