@@ -9,22 +9,25 @@
 #include "wmclient.h"
 #include "wmbutton.h"
 #include "wmoption.h"
-#include "prefs.h"
 #include "WinMgr.h"
 #include "wmmgr.h"
+#include "yicon.h"
 
 class YClientContainer;
 class MiniIcon;
 class TaskBarApp;
+class TrayApp;
 class YFrameTitleBar;
 
-class YFrameWindow: public YWindow, public YActionListener, public YTimerListener, public PopDownListener, public YMsgBoxListener, public ClientData {
+class YFrameWindow: public YWindow, public YActionListener, public YTimerListener, public YPopDownListener, public YMsgBoxListener, public ClientData {
 public:
-    YFrameWindow(YWindow *parent, YFrameClient *client);
+    YFrameWindow(YActionListener *wmActionListener, YWindow *parent);
     virtual ~YFrameWindow();
 
+    void doManage(YFrameClient *client, bool &doActivate, bool &requestFocus);
+    void afterManage();
     void manage(YFrameClient *client);
-    void unmanage();
+    void unmanage(bool reparent = true);
     void sendConfigure();
 
     void createPointerWindows();
@@ -32,14 +35,9 @@ public:
 
     void focus(bool canWarp = false);
     void activate(bool canWarp = false);
+    void activateWindow(bool raise);
 
-    void activateWindow(bool raise) {
-        if (raise)
-            wmRaise();
-        activate(true);
-    }
-
-    virtual void paint(Graphics &g, int x, int y, unsigned int width, unsigned int height);
+    virtual void paint(Graphics &g, const YRect &r);
 
     virtual bool handleKey(const XKeyEvent &key);
     virtual void handleButton(const XButtonEvent &button);
@@ -83,6 +81,14 @@ public:
     void wmOccupyOnlyWorkspace(long workspace);
     void wmMoveToWorkspace(long workspace);
     void wmSetLayer(long layer);
+#ifdef CONFIG_TRAY
+    void wmSetTrayOption(long option);
+    void wmToggleTray();
+#endif
+#if DO_NOT_COVER_OLD
+    void wmToggleDoNotCover();
+#endif
+    void wmToggleFullscreen();
 
     void minimizeTransients();
     void restoreMinimizedTransients();
@@ -94,12 +100,17 @@ public:
     void loseWinFocus();
     void setWinFocus();
     bool focused() const { return fFocused; }
-    void focusOnMap();
+    void updateFocusOnMap(bool &doActivate);
 
     YFrameClient *client() const { return fClient; }
     YFrameTitleBar *titlebar() const { return fTitleBar; }
     YClientContainer *container() const { return fClientContainer; }
 
+#ifdef WMSPEC_HINTS
+    void startMoveSize(int x, int y,
+                                     int direction);
+#endif
+
     void startMoveSize(int doMove, int byMouse,
                        int sideX, int sideY,
                        int mouseXroot, int mouseYroot);
@@ -111,7 +122,7 @@ public:
                 int &flags);
     void snapTo(int &wx, int &wy);
 
-    void drawOutline(int x, int y, int w, int h);
+    void drawMoveSizeFX(int x, int y, int w, int h, bool interior = true);
     int handleMoveKeys(const XKeyEvent &xev, int &newX, int &newY);
     int handleResizeKeys(const XKeyEvent &key,
                          int &newX, int &newY, int &newWidth, int &newHeight,
@@ -126,7 +137,7 @@ public:
     void constrainPositionByModifier(int &x, int &y, const XMotionEvent &motion);
     void constrainMouseToWorkspace(int &x, int &y);
 
-    void getDefaultOptions();
+    void getDefaultOptions(bool &doActivate);
 
     bool canSize(bool boriz = true, bool vert = true);
     bool canMove();
@@ -137,9 +148,10 @@ public:
     bool canHide();
     bool canLower();
     bool canRaise();
+    bool canFullscreen() { return true; }
     bool Overlaps(bool below);
 
-    void insertFrame();
+    void insertFrame(bool top);
     void removeFrame();
     void setAbove(YFrameWindow *aboveFrame); // 0 = at the bottom
     void setBelow(YFrameWindow *belowFrame); // 0 = at the top
@@ -178,19 +190,20 @@ public:
     virtual void raise();
     virtual void lower();
 
-    void popupSystemMenu(int x, int y,
-                         int x_delta, int y_delta,
+    void popupSystemMenu(YWindow *owner, int x, int y,
                          unsigned int flags,
                          YWindow *forWindow = 0);
-    virtual void popupSystemMenu(void);
+    virtual void popupSystemMenu(YWindow *owner);
     virtual void handlePopDown(YPopupWindow *popup);
 
-    virtual void configure(int x, int y, unsigned int width, unsigned int height);
+    virtual void configure(const YRect &r);
     
+    void getNewPos(const XConfigureRequestEvent &cr,
+                   int &cx, int &cy, int &cw, int &ch);
     void configureClient(const XConfigureRequestEvent &configureRequest);
     void configureClient(int cx, int cy, int cwidth, int cheight);
 
-#ifdef SHAPE
+#ifdef CONFIG_SHAPE
     void setShape();
 #endif
 
@@ -217,20 +230,37 @@ public:
         fdDepth         = (1 << 9)
     } YFrameDecors;
 
+    /// !!! needs refactoring (some are not optional right now)
+    /// should be #ifndef NO_WINDOW_OPTIONS
+
     enum YFrameOptions {
-        foAllWorkspaces = (1 << 1),
-        foIgnoreTaskBar = (1 << 2),
-        foIgnoreWinList = (1 << 3),
-        foFullKeys      = (1 << 4),
-        foIgnoreQSwitch = (1 << 5),
-        foNoFocusOnAppRaise = (1 << 6),
-        foIgnoreNoFocusHint = (1 << 7),
-        foIgnorePosition = (1 << 7)
+        foAllWorkspaces         = (1 << 0),
+        foIgnoreTaskBar         = (1 << 1),
+        foIgnoreWinList         = (1 << 2),
+        foFullKeys              = (1 << 3),
+        foIgnoreQSwitch         = (1 << 4),
+        foNoFocusOnAppRaise     = (1 << 5),
+        foIgnoreNoFocusHint     = (1 << 6),
+        foIgnorePosition        = (1 << 7),
+        foDoNotCover            = (1 << 8),
+        foFullscreen            = (1 << 9),
+        foMaximizedVert         = (1 << 10),
+        foMaximizedHorz         = (1 << 11),
+        foNonICCCMConfigureRequest = (1 << 12),
+        foMinimized             = (1 << 13),
+        foDoNotFocus            = (1 << 14),
+        foForcedClose           = (1 << 15),
+        foNoFocusOnMap          = (1 << 16),
+        foNoIgnoreTaskBar       = (1 << 17),
+        foAppTakesFocus         = (1 << 18),
+        foIgnoreUrgent          = (1 << 19)
     };
 
     unsigned long frameFunctions() const { return fFrameFunctions; }
     unsigned long frameDecors() const { return fFrameDecors; }
     unsigned long frameOptions() const { return fFrameOptions; }
+    void updateAllowed();
+    void updateNetWMState();
     void getFrameHints();
 #ifndef NO_WINDOW_OPTIONS
     void getWindowOptions(WindowOption &opt, bool remove); /// !!! fix kludges
@@ -239,30 +269,28 @@ public:
 
     YMenu *windowMenu();
 
-    unsigned int borderX() const {
-        return
-            (frameDecors() & fdBorder) ?
-            ((frameDecors() & fdResize) ? wsBorderX : wsDlgBorderX) : 0;
-    }
-    unsigned int borderY() const {
-        return
-            (frameDecors() & fdBorder) ?
-            ((frameDecors() & fdResize) ? wsBorderY : wsDlgBorderY) : 0;
-    }
-    unsigned int titleY() const {
-        return (frameDecors() & fdTitleBar) ? wsTitleBar : 0;
-    }
+    long getState() const { return fWinState; }
+    void setState(long mask, long state);
+
+    bool isFullscreen() const { return (getState() & WinStateFullscreen) ? true : false; }
+
+    int borderXN() const;
+    int borderYN() const;
+    int titleYN() const;
+
+    int borderX() const;
+    int borderY() const;
+    int titleY() const;
     
     void layoutTitleBar();
     void layoutButtons();
     void layoutResizeIndicators();
+    void layoutShape();
     void layoutClient();
 
     //void workspaceShow();
     //void workspaceHide();
 
-    void addToMenu(YMenu *menu);
-
     YFrameWindow *nextLayer();
     YFrameWindow *prevLayer();
 #ifdef CONFIG_WINLIST
@@ -281,13 +309,21 @@ public:
     YFrameWindow *transient() const { return fTransient; }
     YFrameWindow *nextTransient() const { return fNextTransient; }
     YFrameWindow *owner() const { return fOwner; }
+    YFrameWindow *mainOwner();
 
 #ifndef LITE
-    YIcon *getClientIcon() const { return fFrameIcon; }
-    YIcon *clientIcon();
+    ref<YIcon> getClientIcon() const { return fFrameIcon; }
+    ref<YIcon> clientIcon() const;
 #endif
 
-    void getNormalGeometry(int *x, int *y, int *w, int *h);
+    void getNormalGeometryInner(int *x, int *y, int *w, int *h);
+    void setNormalGeometryOuter(int x, int y, int w, int h);
+    void setNormalPositionOuter(int x, int y);
+    void setNormalGeometryInner(int x, int y, int w, int h);
+    void updateDerivedSize(long flagmask);
+
+    void setCurrentGeometryOuter(YRect newSize);
+    void setCurrentPositionOuter(int x, int y);
     void updateNormalSize();
 
     void updateTitle();
@@ -296,8 +332,11 @@ public:
     void updateIcon();
 #endif
     void updateState();
+    void updateLayer(bool restack = true);
     //void updateWorkspace();
     void updateLayout();
+    void updateExtents();
+    void performLayout();
 
     void updateMwmHints();
     void updateProperties();
@@ -305,13 +344,36 @@ public:
     void updateTaskBar();
 #endif
 
+    enum WindowType {
+        wtCombo,
+        wtDesktop,
+        wtDialog,
+        wtDND,
+        wtDock,
+        wtDropdownMenu,
+        wtMenu,
+        wtNormal,
+        wtNotification,
+        wtPopupMenu,
+        wtSplash,
+        wtToolbar,
+        wtTooltip,
+        wtUtility
+    };
+
+    void setWindowType(enum WindowType winType) { fWindowType = winType; }
+    bool isTypeDock(void) { return (fWindowType == wtDock); }
+
     long getWorkspace() const { return fWinWorkspace; }
     void setWorkspace(long workspace);
-    long getLayer() const { return fWinLayer; }
-    void setLayer(long layer);
-    long getState() const { return fWinState; }
-    void setState(long mask, long state);
-
+    void setWorkspaceHint(long workspace);
+    long getActiveLayer() const { return fWinActiveLayer; }
+    void setRequestedLayer(long layer);
+#ifdef CONFIG_TRAY
+    long getTrayOption() const { return fWinTrayOption; }
+    void setTrayOption(long option);
+#endif
+    void setDoNotCover(bool flag);
     bool isMaximized() const { return (getState() &
                                  (WinStateMaximizedHoriz |
                                   WinStateMaximizedVert)) ? true : false; }
@@ -320,6 +382,8 @@ public:
     bool isMaximizedFully() const { return isMaximizedVert() && isMaximizedHoriz(); }
     bool isMinimized() const { return (getState() & WinStateMinimized) ? true : false; }
     bool isHidden() const { return (getState() & WinStateHidden) ? true : false; }
+    bool isSkipPager() const { return (getState() & WinStateSkipPager) ? true : false; }
+    bool isSkipTaskBar() const { return (getState() & WinStateSkipTaskBar) ? true : false; }
     bool isRollup() const { return (getState() & WinStateRollup) ? true : false; }
     bool isSticky() const { return (getState() & WinStateAllWorkspaces) ? true : false; }
     //bool isHidWorkspace() { return (getState() & WinStateHidWorkspace) ? true : false; }
@@ -328,7 +392,7 @@ public:
     bool wasMinimized() const { return (getState() & WinStateWasMinimized) ? true : false; }
     bool wasHidden() const { return (getState() & WinStateWasHidden) ? true : false; }
 
-    bool isIconic() const { return isMinimized() && minimizeToDesktop && fMiniIcon; }
+    bool isIconic() const { return isMinimized() && fMiniIcon; }
 
     MiniIcon *getMiniIcon() const { return fMiniIcon; }
 
@@ -337,24 +401,72 @@ public:
 
     void setSticky(bool sticky);
 
-    int visibleOn(long workspace) {
+    bool visibleOn(long workspace) const {
         return (isSticky() || getWorkspace() == workspace);
     }
-    int visibleNow() { return visibleOn(manager->activeWorkspace()); }
+    bool visibleNow() const { return visibleOn(manager->activeWorkspace()); }
 
     bool isModal();
     bool hasModal();
-    bool isFocusable();
+    bool canFocus();
+    bool canFocusByMouse();
+    bool avoidFocus();
+    bool getInputFocusHint();
+
+    bool inWorkArea() const;
+    bool affectsWorkArea() const;
+
+    bool doNotCover() const {
+        return (frameOptions() & foDoNotCover) ? true : false;
+    }
 
 #ifndef LITE
-    virtual YIcon *getIcon() { return clientIcon(); }
+    virtual ref<YIcon> getIcon() const { return clientIcon(); }
 #endif
-    virtual const char *getTitle() { return client()->windowTitle(); }
-    virtual const char *getIconTitle() { return client()->iconTitle(); }
+
+    virtual ustring getTitle() const { return client()->windowTitle(); }
+    virtual ustring getIconTitle() const { return client()->iconTitle(); }
 
     YFrameButton *getButton(char c);
     void positionButton(YFrameButton *b, int &xPos, bool onRight);
     bool isButton(char c);
+
+#ifdef WMSPEC_HINTS
+    void updateNetWMStrut();
+    void updateNetWMStrutPartial();
+    void updateNetStartupId();
+    void updateNetWMUserTime();
+    void updateNetWMUserTimeWindow();
+    void updateNetWMFullscreenMonitors(int, int, int, int);
+#endif
+    int strutLeft() { return fStrutLeft; }
+    int strutRight() { return fStrutRight; }
+    int strutTop() { return fStrutTop; }
+    int strutBottom() { return fStrutBottom; }
+
+    YFrameWindow *nextCreated() { return fNextCreatedFrame; }
+    YFrameWindow *prevCreated() { return fPrevCreatedFrame; }
+    void setNextCreated(YFrameWindow *f) { fNextCreatedFrame = f; }
+    void setPrevCreated(YFrameWindow *f) { fPrevCreatedFrame = f; }
+
+    YFrameWindow *nextFocus() { return fNextFocusFrame; }
+    YFrameWindow *prevFocus() { return fPrevFocusFrame; }
+    void setNextFocus(YFrameWindow *f) { fNextFocusFrame = f; }
+    void setPrevFocus(YFrameWindow *f) { fPrevFocusFrame = f; }
+
+    void insertFocusFrame(bool focus);
+    void insertLastFocusFrame();
+    void removeFocusFrame();
+
+    void updateUrgency();
+    void setWmUrgency(bool wmUrgency);
+    bool isUrgent() { return fWmUrgency || fClientUrgency; }
+
+    int getScreen();
+
+    long getOldLayer() { return fOldLayer; }
+    void saveOldLayer() { fOldLayer = fWinActiveLayer; }
+
 private:
     /*typedef enum {
         fsMinimized       = 1 << 0,
@@ -369,8 +481,10 @@ private:
     unsigned long fFrameDecors;
     unsigned long fFrameOptions;
 
-    int normalX, normalY;
-    unsigned int normalWidth, normalHeight;
+    int normalX, normalY, normalW, normalH;
+    int posX, posY, posW, posH;
+    int extentLeft, extentRight, extentTop, extentBottom;
+
     int iconX, iconY;
 
     YFrameClient *fClient;
@@ -395,65 +509,104 @@ private:
     YFrameWindow *fNextFrame; // window below this one
     YFrameWindow *fPrevFrame; // window above this one
 
+    YFrameWindow *fNextCreatedFrame;
+    YFrameWindow *fPrevCreatedFrame;
+
+    YFrameWindow *fNextFocusFrame;
+    YFrameWindow *fPrevFocusFrame;
+
     Window topSide, leftSide, rightSide, bottomSide;
     Window topLeftCorner, topRightCorner, bottomLeftCorner, bottomRightCorner;
     int indicatorsVisible;
 
 #ifdef CONFIG_TASKBAR
     TaskBarApp *fTaskBarApp;
+#endif
+#ifdef CONFIG_TRAY
+    TrayApp *fTrayApp;
 #endif
     MiniIcon *fMiniIcon;
 #ifdef CONFIG_WINLIST
     WindowListItem *fWinListItem;
 #endif
-    YIcon *fFrameIcon;
+#ifndef LITE
+    ref<YIcon> fFrameIcon;
+#endif
 
     YFrameWindow *fOwner;
     YFrameWindow *fTransient;
     YFrameWindow *fNextTransient;
+    YActionListener *wmActionListener;
 
     static YTimer *fAutoRaiseTimer;
     static YTimer *fDelayFocusTimer;
 
     long fWinWorkspace;
-    long fWinLayer;
+    long fWinRequestedLayer;
+    long fWinActiveLayer;
+#ifdef CONFIG_TRAY
+    long fWinTrayOption;
+#endif
     long fWinState;
     long fWinStateMask;
     bool fManaged;
+    long fWinOptionMask;
+    long fOldLayer;
+
+    int fFullscreenMonitorsTop;
+    int fFullscreenMonitorsBottom;
+    int fFullscreenMonitorsLeft;
+    int fFullscreenMonitorsRight;
 
     YMsgBox *fKillMsgBox;
+
+    // _NET_WM_STRUT support
+    int fStrutLeft;
+    int fStrutRight;
+    int fStrutTop;
+    int fStrutBottom;
+
+    // _NET_WM_USER_TIME support
+    Time fUserTime;
+    Window fUserTimeWindow;
+
+    int fShapeWidth;
+    int fShapeHeight;
+    int fShapeTitleY;
+    int fShapeBorderX;
+    int fShapeBorderY;
+
+    bool fWmUrgency;
+    bool fClientUrgency;
+
+    enum WindowType fWindowType;
+
+    enum WindowArranges {
+        waTop,
+        waBottom,
+        waCenter,
+        waLeft,
+        waRight
+    };
+
+    void wmArrange(int tcb, int lcr);
+    void wmSnapMove(int tcb, int lcr);
+    int getTopCoord(int my, YFrameWindow **w, int count);
+    int getBottomCoord(int My, YFrameWindow **w, int count);
+    int getLeftCoord(int mx, YFrameWindow **w, int count);
+    int getRightCoord(int Mx, YFrameWindow **w, int count);
+
+    // only focus if mouse moves
+    //static int fMouseFocusX, fMouseFocusY;
+
+    void setGeometry(const YRect &);
+    void setPosition(int, int);
+    void setSize(int, int);
+    void setWindowGeometry(const YRect &r) {
+        YWindow::setGeometry(r);
+    }
+    friend class MiniIcon;
 };
 
-//!!! remove this
-#ifdef CONFIG_LOOK_PIXMAP
-extern YPixmap *frameTL[2][2];
-extern YPixmap *frameT[2][2];
-extern YPixmap *frameTR[2][2];
-extern YPixmap *frameL[2][2];
-extern YPixmap *frameR[2][2];
-extern YPixmap *frameBL[2][2];
-extern YPixmap *frameB[2][2];
-extern YPixmap *frameBR[2][2];
-
-extern YPixmap *titleL[2];
-extern YPixmap *titleS[2];
-extern YPixmap *titleP[2];
-extern YPixmap *titleT[2];
-extern YPixmap *titleM[2];
-extern YPixmap *titleB[2];
-extern YPixmap *titleR[2];
-
-extern YPixmap *menuButton[2];
 #endif
 
-extern Cursor movePointer;
-extern Cursor sizeRightPointer;
-extern Cursor sizeTopRightPointer;
-extern Cursor sizeTopPointer;
-extern Cursor sizeTopLeftPointer;
-extern Cursor sizeLeftPointer;
-extern Cursor sizeBottomLeftPointer;
-extern Cursor sizeBottomPointer;
-extern Cursor sizeBottomRightPointer;
-
-#endif
