@@ -4,6 +4,7 @@
  * Copyright (C) 1997-2001 Marko Macek
  */
 #include "config.h"
+#include "globit.h"
 
 #ifndef LITE
 #include "ykey.h"
@@ -11,14 +12,16 @@
 #include "ymenu.h"
 #include "ymenuitem.h"
 
-#include "yapp.h"
+#include "yxapp.h"
 #include "prefs.h"
 
 #include <string.h>
+#include <stdlib.h>
+#include <limits.h>
 
 #include "intl.h"
 
-YFont *YInputLine::inputFont = 0;
+ref<YFont> YInputLine::inputFont;
 YColor *YInputLine::inputBg = 0;
 YColor *YInputLine::inputFg = 0;
 YColor *YInputLine::inputSelectionBg = 0;
@@ -30,9 +33,9 @@ int YInputLine::fAutoScrollDelta = 0;
 
 static YAction *actionCut, *actionCopy, *actionPaste, *actionSelectAll, *actionPasteSelection;
 
-YInputLine::YInputLine(YWindow *parent): YWindow(parent) {
-    if (inputFont == 0)
-        inputFont = YFont::getFont(inputFontName);
+YInputLine::YInputLine(YWindow *parent): YWindow(parent), fText(null) {
+    if (inputFont == null)
+        inputFont = YFont::getFont(XFA(inputFontName));
     if (inputBg == 0)
         inputBg = new YColor(clrInput);
     if (inputFg == 0)
@@ -53,20 +56,19 @@ YInputLine::YInputLine(YWindow *parent): YWindow(parent) {
             inputMenu->addItem(_("Cu_t"), -2, _("Ctrl+X"), actionCut)->setEnabled(true);
             inputMenu->addItem(_("_Copy"), -2, _("Ctrl+C"), actionCopy)->setEnabled(true);
             inputMenu->addItem(_("_Paste"), -2, _("Ctrl+V"), actionPaste)->setEnabled(true);
-            inputMenu->addItem(_("Paste _Selection"), -2, 0, actionPasteSelection)->setEnabled(true);
+            inputMenu->addItem(_("Paste _Selection"), -2, null, actionPasteSelection)->setEnabled(true);
             inputMenu->addSeparator();
             inputMenu->addItem(_("Select _All"), -2, _("Ctrl+A"), actionSelectAll);
         }
     }
 
-    fText = 0;
     curPos = 0;
     markPos = 0;
     leftOfs = 0;
     fHasFocus = false;
     fSelecting = false;
     fCursorVisible = true;
-    if (inputFont)
+    if (inputFont != null)
         setSize(width(), inputFont->height() + 2);
 }
 YInputLine::~YInputLine() {
@@ -76,27 +78,24 @@ YInputLine::~YInputLine() {
             cursorBlinkTimer->setTimerListener(0);
         }
     }
-    delete fText; fText = 0;
 }
 
-void YInputLine::setText(const char *text) {
-    delete fText;
-    fText = newstr(text);
+void YInputLine::setText(const ustring &text) {
+    fText = text;
     markPos = curPos = leftOfs = 0;
-    if (fText)
-        curPos = strlen(fText);
+    curPos = fText.length();
     limit();
     repaint();
 }
 
-const char *YInputLine::getText() {
+ustring YInputLine::getText() {
     return fText;
 }
 
-void YInputLine::paint(Graphics &g, int /*x*/, int /*y*/, unsigned int /*width*/, unsigned int /*height*/) {
-    YFont *font = inputFont;
+void YInputLine::paint(Graphics &g, const YRect &/*r*/) {
+    ref<YFont> font = inputFont;
     int min, max, minOfs = 0, maxOfs = 0;
-    int textLen = fText ? strlen(fText) : 0;
+    int textLen = fText.length();
 
     if (curPos > markPos) {
         min = markPos;
@@ -106,12 +105,12 @@ void YInputLine::paint(Graphics &g, int /*x*/, int /*y*/, unsigned int /*width*/
         max = markPos;
     }
 
-    if (curPos == markPos || !fText || !font || !fHasFocus) {
+    if (curPos == markPos || fText == null || font == null || !fHasFocus) {
         g.setColor(inputBg);
         g.fillRect(0, 0, width(), height());
     } else {
-        minOfs = font->textWidth(fText, min) - leftOfs;
-        maxOfs = font->textWidth(fText, max) - leftOfs;
+        minOfs = font->textWidth(fText.substring(0, min)) - leftOfs;
+        maxOfs = font->textWidth(fText.substring(0, max)) - leftOfs;
 
         if (minOfs > 0) {
             g.setColor(inputBg);
@@ -128,32 +127,32 @@ void YInputLine::paint(Graphics &g, int /*x*/, int /*y*/, unsigned int /*width*/
         }
     }
 
-    if (font) {
+    if (font != null) {
         int yp = 1 + font->ascent();
-        int curOfs = fText ? font->textWidth(fText, curPos) : 0;
+        int curOfs = font->textWidth(fText.substring(0, curPos));
         int cx = curOfs - leftOfs;
 
         g.setFont(font);
 
-        if (curPos == markPos || !fHasFocus || !fText) {
+        if (curPos == markPos || !fHasFocus || fText == null) {
             g.setColor(inputFg);
-            if (fText)
-                g.drawChars(fText, 0, textLen, -leftOfs, yp);
+            if (fText != null)
+                g.drawChars(fText.substring(0, textLen), -leftOfs, yp);
             if (fHasFocus && fCursorVisible)
                 g.drawLine(cx, 0, cx, font->height() + 2);
         } else {
             if (min > 0) {
                 g.setColor(inputFg);
-                g.drawChars(fText, 0, min, -leftOfs, yp);
+                g.drawChars(fText.substring(0, min), -leftOfs, yp);
             }
             /// !!! same here
             if (min < max) {
                 g.setColor(inputSelectionFg);
-                g.drawChars(fText, min, max - min, minOfs, yp);
+                g.drawChars(fText.substring(min, max - min), minOfs, yp);
             }
             if (max < textLen) {
                 g.setColor(inputFg);
-                g.drawChars(fText, max, textLen - max, maxOfs, yp);
+                g.drawChars(fText.substring(max, textLen - max), maxOfs, yp);
             }
         }
     }
@@ -161,10 +160,29 @@ void YInputLine::paint(Graphics &g, int /*x*/, int /*y*/, unsigned int /*width*/
 
 bool YInputLine::handleKey(const XKeyEvent &key) {
     if (key.type == KeyPress) {
-        KeySym k = XKeycodeToKeysym(app->display(), key.keycode, 0);
+        KeySym k = keyCodeToKeySym(key.keycode);
+
+        switch (k) {
+        case XK_KP_Home:
+        case XK_KP_Up:
+        case XK_KP_Prior:
+        case XK_KP_Left:
+        case XK_KP_Begin:
+        case XK_KP_Right:
+        case XK_KP_End:
+        case XK_KP_Down:
+        case XK_KP_Next:
+        case XK_KP_Insert:
+        case XK_KP_Delete:
+            if (key.state & xapp->NumLockMask) {
+                k = keyCodeToKeySym(key.keycode, 1);
+            }
+            break;
+        }
+
         int m = KEY_MODMASK(key.state);
         bool extend = (m & ShiftMask) ? true : false;
-        int textLen = fText ? strlen(fText) : 0;
+        int textLen = fText.length();
 
         if (m & ControlMask) {
             switch(k) {
@@ -191,6 +209,10 @@ bool YInputLine::handleKey(const XKeyEvent &key) {
             case XK_KP_Insert:
                 copySelection();
                 return true;
+	    case 'i':
+	    case 'I':
+		complete();
+		return true;
             }
         }
         if (m & ShiftMask) {
@@ -284,13 +306,16 @@ bool YInputLine::handleKey(const XKeyEvent &key) {
                 }
             }
             break;
+	case XK_Tab:
+	    complete();
+	    break;
         default:
             {
-                char c;
+                char s[16];
 
-                if (getCharFromEvent(key, &c)) {
-                    if (insertChar(c))
-                        return true;
+                if (getCharFromEvent(key, s, sizeof(s))) {
+                    replaceSelection(ustring(s, strlen(s)));
+                    return true;
                 }
             }
         }
@@ -365,8 +390,7 @@ void YInputLine::handleClickDown(const XButtonEvent &down, int count) {
             }
         } else if ((count % 4) == 3) {
             markPos = curPos = 0;
-            if (fText)
-                curPos = strlen(fText);
+            curPos = fText.length();
             fSelecting = false;
             limit();
             repaint();
@@ -377,7 +401,7 @@ void YInputLine::handleClickDown(const XButtonEvent &down, int count) {
 void YInputLine::handleClick(const XButtonEvent &up, int /*count*/) {
     if (up.button == 3 && IS_BUTTON(up.state, Button3Mask)) {
         if (inputMenu)
-            inputMenu->popup(0, 0, up.x_root, up.y_root, -1, -1,
+            inputMenu->popup(this, 0, 0, up.x_root, up.y_root,
                              YPopupWindow::pfCanFlipVertical |
                              YPopupWindow::pfCanFlipHorizontal);
     } else if (up.button == 2 && IS_BUTTON(up.state, Button2Mask)) {
@@ -391,32 +415,35 @@ void YInputLine::handleSelection(const XSelectionEvent &selection) {
         long extra;
         int format;
         long nitems;
-        char *data;
+        union {
+            char *ptr;
+            unsigned char *xptr;
+        } data;
 
-        XGetWindowProperty(app->display(),
+        XGetWindowProperty(xapp->display(),
                            selection.requestor, selection.property,
                            0L, 32 * 1024, True,
                            selection.target, &type, &format,
                            (unsigned long *)&nitems,
                            (unsigned long *)&extra,
-                           (unsigned char **)&data);
+                           &(data.xptr));
 
-        if (nitems > 0 && data != NULL) {
-            replaceSelection(data, nitems);
+        if (nitems > 0 && data.ptr != NULL) {
+            replaceSelection(mstring(data.ptr, nitems));
         }
-        if (data != NULL)
-            XFree(data);
+        if (data.xptr != NULL)
+            XFree(data.xptr);
     }
 }
 
 int YInputLine::offsetToPos(int offset) {
-    YFont *font = inputFont;
+    ref<YFont> font = inputFont;
     int ofs = 0, pos = 0;;
-    int textLen = fText ? strlen(fText) : 0;
+    int textLen = fText.length();
 
-    if (font) {
+    if (font != null) {
         while (pos < textLen) {
-            ofs += font->textWidth(fText + pos, 1);
+            ofs += font->textWidth(fText.substring(pos, 1));
             if (ofs < offset)
                 pos++;
             else
@@ -475,7 +502,7 @@ bool YInputLine::handleTimer(YTimer *t) {
 }
 
 bool YInputLine::move(int pos, bool extend) {
-    int textLen = fText ? strlen(fText) : 0;
+    int textLen = fText.length();
 
     if (curPos < 0 || curPos > textLen)
         return false;
@@ -493,7 +520,7 @@ bool YInputLine::move(int pos, bool extend) {
 }
 
 void YInputLine::limit() {
-    int textLen = fText ? strlen(fText) : 0;
+    int textLen = fText.length();
 
     if (curPos > textLen)
         curPos = textLen;
@@ -504,28 +531,28 @@ void YInputLine::limit() {
     if (markPos < 0)
         markPos = 0;
 
-    YFont *font = inputFont;
-    if (font) {
-        int curOfs = font->textWidth(fText, curPos);
-        int curLen = font->textWidth(fText, textLen);
+    ref<YFont> font = inputFont;
+    if (font != null) {
+        int curOfs = font->textWidth(fText.substring(0, curPos));
+        int curLen = font->textWidth(fText.substring(0, textLen));
 
-        if (curOfs >= leftOfs + int(width()))
-            leftOfs = curOfs - width() + 1;
+        if (curOfs >= leftOfs + int(width()) + 1)
+            leftOfs = curOfs - width() + 2;
         if (curOfs < leftOfs)
             leftOfs = curOfs;
-        if (leftOfs + int(width()) > curLen)
-            leftOfs = curLen - width();
+        if (leftOfs + int(width()) + 1 > curLen)
+            leftOfs = curLen - width() + 1;
         if (leftOfs < 0)
             leftOfs = 0;
     }
 }
 
-void YInputLine::replaceSelection(const char *str, int len) {
-    int newStrLen;
-    char *newStr;
-    int textLen = fText ? strlen(fText) : 0;
+void YInputLine::replaceSelection(const ustring &str) {
     int min, max;
 
+    // FIXME: GCC7 tells us for optimized builds: src/yinput.cc:552:5: warning: assuming signed overflow does not occur when assuming that (X + c) < X is always false [-Wstrict-overflow]
+    // that might imply that all code paths leading to here are already such that the condition always holds true
+    // Unfortunatelly GCC does not help much here.
     if (curPos > markPos) {
         min = markPos;
         max = curPos;
@@ -534,19 +561,11 @@ void YInputLine::replaceSelection(const char *str, int len) {
         max = markPos;
     }
 
-    newStrLen = min + len + (textLen - max);
-    newStr = new char[newStrLen + 1];
-    if (newStr) {
-        if (min)
-            memcpy(newStr, fText, min);
-        if (len)
-            memcpy(newStr + min, str, len);
-        if (max < textLen)
-            memcpy(newStr + min + len, fText + max, textLen - max);
-        newStr[newStrLen] = 0;
-        delete fText;
+    ustring newStr = fText.replace(min, max - min, str);
+
+    if (newStr != null) {
         fText = newStr;
-        curPos = markPos = min + len;
+        curPos = markPos = min + str.length();
         limit();
         repaint();
     }
@@ -554,17 +573,17 @@ void YInputLine::replaceSelection(const char *str, int len) {
 
 bool YInputLine::deleteSelection() {
     if (hasSelection()) {
-        replaceSelection(0, 0);
+        replaceSelection(null);
         return true;
     }
     return false;
 }
 
 bool YInputLine::deleteNextChar() {
-    int textLen = fText ? strlen(fText) : 0;
+    int textLen = fText.length();
 
     if (curPos < textLen) {
-        markPos = curPos + 1;
+        markPos = curPos + (curPos < INT_MAX);
         deleteSelection();
         return true;
     }
@@ -581,18 +600,22 @@ bool YInputLine::deletePreviousChar() {
 }
 
 bool YInputLine::insertChar(char ch) {
-    replaceSelection(&ch, 1);
+    char s[2] = { ch, 0 };
+    replaceSelection(s);
     return true;
 }
 
-#define CHCLASS(c) (((c) == ' ') ? 1 : 0)
+#define CHCLASS(c) ((c) == ' ')
 
 int YInputLine::nextWord(int p, bool sep) {
-    int textLen = fText ? strlen(fText) : 0;
+    int textLen = fText.length();
 
-    while (p < textLen && (CHCLASS(fText[p]) == CHCLASS(fText[p + 1]) ||
-                           !sep && CHCLASS(fText[p]) == 1))
+    while (p < textLen &&
+           (CHCLASS(fText.charAt(p)) == CHCLASS(fText.charAt(p + 1)) ||
+            (!sep && CHCLASS(fText.charAt(p)))))
+    {
         p++;
+    }
     if (p < textLen)
         p++;
     return p;
@@ -601,9 +624,12 @@ int YInputLine::nextWord(int p, bool sep) {
 int YInputLine::prevWord(int p, bool sep) {
     if (p > 0 && !sep)
         p--;
-    while (p > 0 && (CHCLASS(fText[p]) == CHCLASS(fText[p - 1]) ||
-                     !sep && CHCLASS(fText[p]) == 1))
+    while (p > 0 &&
+           (CHCLASS(fText.charAt(p)) == CHCLASS(fText.charAt(p - 1)) ||
+            (!sep && CHCLASS(fText.charAt(p)))))
+    {
         p--;
+    }
     return p;
 }
 
@@ -625,7 +651,7 @@ bool YInputLine::deletePreviousWord() {
 }
 
 bool YInputLine::deleteToEnd() {
-    int textLen = fText ? strlen(fText) : 0;
+    int textLen = fText.length();
 
     if (curPos < textLen) {
         markPos = textLen;
@@ -644,8 +670,7 @@ bool YInputLine::deleteToBegin() {
 
 void YInputLine::selectAll() {
     markPos = curPos = 0;
-    if (fText)
-        curPos = strlen(fText);
+    curPos = fText.length();
     fSelecting = false;
     limit();
     repaint();
@@ -659,7 +684,7 @@ void YInputLine::unselectAll() {
     }
 }
 void YInputLine::cutSelection() {
-    if (!fText)
+    if (fText == null)
         return ;
     if (hasSelection()) {
         copySelection();
@@ -670,7 +695,7 @@ void YInputLine::cutSelection() {
 void YInputLine::copySelection() {
     int min, max;
     if (hasSelection()) {
-        if (!fText)
+        if (fText == null)
             return ;
         if (curPos > markPos) {
             min = markPos;
@@ -679,7 +704,7 @@ void YInputLine::copySelection() {
             min = curPos;
             max = markPos;
         }
-        app->setClipboardText(fText + min, max - min);
+        xapp->setClipboardText(fText.substring(min, max - min));
     }
 }
 
@@ -700,4 +725,21 @@ void YInputLine::autoScroll(int delta, const XMotionEvent *motion) {
     fAutoScrollDelta = delta;
     beginAutoScroll(delta ? true : false, motion);
 }
+void YInputLine::complete() {
+    char *res=NULL;
+    int  res_count=0;
+    cstring t(fText);
+
+    res_count = globit_best(t.c_str(), &res);
+    if (res_count == -1) { //error-case
+	    free(res);
+	    return;
+    }
+    if (res_count == 0) { //no match found
+	    return;
+    }
+    setText(ustring(res, strlen(res)));
+    free(res);
+}
+
 #endif
